
#include "pm.h"
#include "char_font.h"

/* The attribute const puts the variable in ROM of the PM. */
static const short mask[] = { 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80 };

#define draw_pixel(x, y) draw_pixel_asm(x, y, mask);
/* Demo how to create ASM inlined code. */
void draw_pixel_asm(__reg("x") int, __reg("y") int, const short * ) = 
	"\tpush hl\n"\
	"\tmov ba, y\n"\
	"\tand a, 0xf8\n"\
	"\tmov l,12\n"\
	"\tmul l,a\n"\
	"\tadd hl,0x1000\n"\
	"\tadd x,hl\n"\
	"\tmov ba, y\n"\
	"\tand a, 0x07\n"\
	"\tmov b, 0\n"\
	"\tmov hl,[sp+2]\n"\
	"\tadd hl, ba\n"\
	"\tmov a,[hl]\n"\
	"\tmov hl,x\n"\
	"\tor [hl], a\n"\
	"\tpop hl\n";



void print_ch(int x, int y, unsigned char ch)
{
	int i;
	const unsigned short *p2;
	unsigned char *p = (unsigned char *)(0x1000 + x * 6 + y * 96);
		
	if(ch < 0x20)
		ch = 0;
	else
		ch -= 0x20;
	if(ch>=0xa0)
		ch = 0;
	p2 = char_font[ch];
	for(i = 0; i<5; i++)
		*p++ = *p2++;
	*p++ = 0;
}

void print_digit(int x, int y, int digit)
{
	if(0<=digit && digit <=9)
		digit += '0';
	else if(0xA<=digit && digit <=0xF)
		digit += 'A' - 0xA;
	else
		digit = '?';
		
	print_ch(x, y, digit);
}

void print_dec(char x, char y, int num)
{
	int base;
	if(num < 0)
	{
		print_ch(x, y, '-');
		x += 1;
		num = -num;
	}
	for(base = 10000; base > 1; base /= 10)
		if(num >= base)
			break;
	for(; base >= 1; base /= 10)
	{
		print_digit(x, y, num / base);
		num %= base;
		x += 1;
	}
}

void print_hex(char x, char y, unsigned num)
{
	char i;
	for(i=12; i>=0; i-=4, x+=1)
		print_digit(x, y, (num >>i) & 0x0f);
}

void print(int x, int y, const char *str)
{
	for(; *str; ++str)
		print_ch(x++, y, *str);
}

void draw_circle(char xc, char yc, char r)
{
    unsigned int x= r, y= 0;     
    int          cd2= 0;    
	
    if (!r) return; 
    draw_pixel(xc-r, yc);
    draw_pixel(xc+r, yc);
    draw_pixel(xc, yc-r);
    draw_pixel(xc, yc+r);
 
    while (x > y)
    {
        cd2-= (--x) - (++y);
        if (cd2 < 0) cd2+=x++;

        draw_pixel(xc-x, yc-y);
        draw_pixel(xc-y, yc-x);
        draw_pixel(xc+y, yc-x);
        draw_pixel(xc+x, yc-y);
        draw_pixel(xc-x, yc+y);
        draw_pixel(xc-y, yc+x);
        draw_pixel(xc+y, yc+x);
        draw_pixel(xc+x, yc+y);
     } 
}
void clear_screen()
{
	int i;
	char *p = (char *)0x1000;
	for(i=0; i<0x300; i++)
		*p++=0;
}

/* This variable will be accessed from the interrupt routine, so make it volatile. */
volatile int button_a_pressed;

/* Interrupt routines should have __interrupt atribute and contain the interrupt number in the function name */
void __interrupt irq_22_key_a()
{
	IRQ_ACT3 = 0x01;
	
	if((KEY_PAD & KEY_A) == 0)
		button_a_pressed = 1;
}

void wait_key_press()
{
	/* wait for the user to press the "A" button, handled in the interrut routine */
	button_a_pressed = 0;
	while(!button_a_pressed)
		;
}

short __align8 tiles[16 * 12 * 8];

void tile_map_init()
{
	unsigned char *tile_map_mem = (unsigned char *)0x1360;
	unsigned char i, j; 
	char *p = (char *)tiles;
	
	for(i=0; i<16*12; i++)
		tile_map_mem[i] = i;
	for(i=0; i<16*12; i++)
	{
		for(j=0; j<5; j++)
			tiles[8*i + j] = char_font[i][j];
	}
			
	PRC_MAP_LO = ((int)p);
	PRC_MAP_MID = (((int)p) >> 8);
	PRC_MAP_HI = 0x00;
	
	PRC_SCROLL_X = 0;
	PRC_SCROLL_Y = 0;
	PRC_MODE = 0x1a;
}

void tile_map_drive()
{
	int i;
	char x, y, dx, dy;
	
	x = 0; 
	y = 5;
	dx = 1;
	dy = 1;

	button_a_pressed = 0;
	for(;;)
	{
		PRC_SCROLL_X = x;
		PRC_SCROLL_Y = y;
		for(i=0; i<1000; i++)
			;
		x+=dx;
		y+=dy;
		
		if(x==0)
			dx = 1;
		if(x==29)
			dx = -1;
		if(y==0)
			dy = 1;
		if(y==15)
			dy = -1;

		if(button_a_pressed)
		{
			wait_key_press();
			button_a_pressed = 0;
		}
	}
}

int main()
{
	int i, a;
	char *p;	
		
	IRQ_PRI2 = 0x0c;
	IRQ_ENA3 = 0x81;

	print(2,3, "Hello world!");
	print(2,5, " press <A>");
	
	wait_key_press();
	
	clear_screen();
	for(i=0; i<=0xf; i++)
		print_digit(i, 6, i);
	for(a = 0; a <80; a++)
		draw_pixel(a+10, 5 +(a>>1));
	draw_circle(25, 20, 15);
	draw_circle(35, 25, 21);
	
	wait_key_press();
		
	draw_circle(63, 30, 29);
	draw_circle(63, 30, 26);
	
	wait_key_press();
	
	clear_screen();
	
	tile_map_init();
	tile_map_drive();

	return 0;
}
